"""Prompt templates for AI-powered debugging.

This module provides structured prompt templates for various
AI-powered debugging tasks.
"""

from autodebug.prompts.templates.fix_generation import (
    CONSERVATIVE_FIX_PROMPT,
    CREATIVE_FIX_PROMPT,
    ENSEMBLE_FIX_PROMPT,
    FIX_GENERATION_SYSTEM_PROMPT,
    FRAMEWORK_FIX_PROMPT,
    RAG_BASED_FIX_PROMPT,
    TEMPLATE_FIX_PROMPT,
    VALIDATE_FIX_PROMPT,
    format_conservative_fix_prompt,
    format_creative_fix_prompt,
    format_ensemble_fix_prompt,
    format_framework_fix_prompt,
    format_rag_fix_prompt,
    format_validation_prompt,
)
from autodebug.prompts.templates.pr_review import (
    GITHUB_COMMENT_PROMPT,
    PATTERN_DETECTION_PROMPT,
    PR_REVIEW_PROMPT,
    PR_REVIEW_SYSTEM_PROMPT,
    REVIEW_SUMMARY_PROMPT,
    TEST_FILE_REVIEW_PROMPT,
    format_github_comment_prompt,
    format_pattern_detection_prompt,
    format_pr_review_prompt,
    format_review_summary_prompt,
    format_test_file_review_prompt,
)
from autodebug.prompts.templates.root_cause import (
    CATEGORIZE_FLAKINESS_PROMPT,
    EXPLAIN_ROOT_CAUSE_PROMPT,
    GENERATE_FIX_RECOMMENDATIONS_PROMPT,
    IDENTIFY_DIVERGENCE_PROMPT,
    ROOT_CAUSE_COT_PROMPT,
    ROOT_CAUSE_SYSTEM_PROMPT,
    format_categorize_prompt,
    format_explanation_prompt,
    format_fix_recommendations_prompt,
    format_root_cause_prompt,
)
from autodebug.prompts.templates.test_generation import (
    GENERATE_PYTEST_PROMPT,
    HTTP_FIXTURE_PROMPT,
    PBT_GENERATION_PROMPT,
    REDIS_FIXTURE_PROMPT,
    SQL_FIXTURE_PROMPT,
    TEST_GENERATION_SYSTEM_PROMPT,
    TEST_NAME_PROMPT,
    format_http_fixture_prompt,
    format_pbt_prompt,
    format_pytest_generation_prompt,
    format_redis_fixture_prompt,
    format_sql_fixture_prompt,
    format_test_name_prompt,
)

__all__ = [
    # Root cause analysis
    "ROOT_CAUSE_SYSTEM_PROMPT",
    "ROOT_CAUSE_COT_PROMPT",
    "CATEGORIZE_FLAKINESS_PROMPT",
    "IDENTIFY_DIVERGENCE_PROMPT",
    "EXPLAIN_ROOT_CAUSE_PROMPT",
    "GENERATE_FIX_RECOMMENDATIONS_PROMPT",
    "format_root_cause_prompt",
    "format_categorize_prompt",
    "format_explanation_prompt",
    "format_fix_recommendations_prompt",
    # Fix generation
    "FIX_GENERATION_SYSTEM_PROMPT",
    "CONSERVATIVE_FIX_PROMPT",
    "RAG_BASED_FIX_PROMPT",
    "FRAMEWORK_FIX_PROMPT",
    "CREATIVE_FIX_PROMPT",
    "TEMPLATE_FIX_PROMPT",
    "ENSEMBLE_FIX_PROMPT",
    "VALIDATE_FIX_PROMPT",
    "format_conservative_fix_prompt",
    "format_rag_fix_prompt",
    "format_framework_fix_prompt",
    "format_creative_fix_prompt",
    "format_ensemble_fix_prompt",
    "format_validation_prompt",
    # Test generation
    "TEST_GENERATION_SYSTEM_PROMPT",
    "GENERATE_PYTEST_PROMPT",
    "HTTP_FIXTURE_PROMPT",
    "SQL_FIXTURE_PROMPT",
    "REDIS_FIXTURE_PROMPT",
    "PBT_GENERATION_PROMPT",
    "TEST_NAME_PROMPT",
    "format_pytest_generation_prompt",
    "format_http_fixture_prompt",
    "format_sql_fixture_prompt",
    "format_redis_fixture_prompt",
    "format_pbt_prompt",
    "format_test_name_prompt",
    # PR review
    "PR_REVIEW_SYSTEM_PROMPT",
    "PR_REVIEW_PROMPT",
    "TEST_FILE_REVIEW_PROMPT",
    "GITHUB_COMMENT_PROMPT",
    "REVIEW_SUMMARY_PROMPT",
    "PATTERN_DETECTION_PROMPT",
    "format_pr_review_prompt",
    "format_test_file_review_prompt",
    "format_github_comment_prompt",
    "format_review_summary_prompt",
    "format_pattern_detection_prompt",
]
